package org.json;

import static org.junit.Assert.*;

import java.util.Calendar;
import java.util.Date;
import java.util.TimeZone;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

public class TestJSONArray
{

    @Before
    public void setUp() throws Exception
    {
    }

    @After
    public void tearDown() throws Exception
    {
    }


    @Test
    public void testJSONArrayConstruct()
    {
        
    }

    @Test
    public void testJSONArrayGetSet()
    {
        JSONArray ja = new JSONArray();

        ja.put(0, (Object)null);
        Object o = ja.get(0);
        assertNull(o);

        // int
        JSONArray t = ja.put(1, 0);
        assertEquals(t, ja);
        assertTrue(t == ja);
        ja.put(2, -39543);
        int i = ja.getInt(1);
        assertEquals(0, i);
        i = ja.getInt(2);
        assertEquals(-39543, i);
        try
        {
            i = ja.getInt(33);
            fail("exception expected");
        }
        catch (Exception ex)
        {
            assertTrue(ex instanceof IndexOutOfBoundsException);
        }
        try
        {
            i = ja.getInt(0);
            fail("exception expected");
        }
        catch (Exception ex)
        {
            assertTrue(ex instanceof JSONException);
        }

        // boolean
        ja.put(3, true);
        ja.put(4, false);
        boolean b = ja.getBoolean(3);
        assertEquals(true, b);
        b = ja.getBoolean(4);
        assertEquals(false, b);
        try
        {
            b = ja.getBoolean(33);
            fail("exception expected");
        }
        catch (Exception ex)
        {
            assertTrue(ex instanceof IndexOutOfBoundsException);
        }
        try
        {
            b = ja.getBoolean(0);
            fail("exception expected");
        }
        catch (Exception ex)
        {
            assertTrue(ex instanceof JSONException);
        }

        // double
        ja.put(5, -2.87e15);
        ja.put(3.9942);
        double d = ja.getDouble(5);
        assertEquals(-2.87e15, d);
        d = ja.getDouble(6);
        assertEquals(3.9942, d);
        try
        {
            d = ja.getDouble(33);
            fail("exception expected");
        }
        catch (Exception ex)
        {
            assertTrue(ex instanceof IndexOutOfBoundsException);
        }
        try
        {
            d = ja.getDouble(0);
            fail("exception expected");
        }
        catch (Exception ex)
        {
            assertTrue(ex instanceof JSONException);
        }
        // bad numbers
        try {
            double bad = Double.NaN;
            ja.put(bad);
            fail("exception expected");
        }
        catch (Exception ex)
        {
            assertTrue(ex instanceof JSONException);
        }
        try {
            Float bad = Float.NEGATIVE_INFINITY;
            ja.put(bad);
            fail("exception expected");
        }
        catch (Exception ex)
        {
            assertTrue(ex instanceof JSONException);
        }

        // long
        ja.put(7, 990001L);
        ja.put(8, -23L);
        long l = ja.getLong(7);
        assertEquals(990001L, l);
        l = ja.getLong(8);
        assertEquals(-23L, l);
        try
        {
            l = ja.getLong(33);
            fail("exception expected");
        }
        catch (Exception ex)
        {
            assertTrue(ex instanceof IndexOutOfBoundsException);
        }
        try
        {
            l = ja.getLong(0);
            fail("exception expected");
        }
        catch (Exception ex)
        {
            assertTrue(ex instanceof JSONException);
        }

        // String
        ja.put(9, "abc");
        String s = ja.getString(9);
        assertEquals("abc", s);
        s = ja.getString(0);
        assertEquals("null", s);

        try
        {
            l = ja.getLong(33);
            fail("exception expected");
        }
        catch (Exception ex)
        {
            assertTrue(ex instanceof IndexOutOfBoundsException);
        }

        // Date
        Calendar calendar = Calendar.getInstance();
        calendar.clear();
        calendar.set(2006, 1, 2, 11, 32);
        Date d1 = calendar.getTime();
        Date now = new Date();

        ja.put(10, d1);
        Date date = ja.getDate(10);
        assertEquals(d1, date);
        try
        {
            date = ja.getDate(42);
            fail("exception expected");
        }
        catch (Exception ex)
        {
            assertTrue(ex instanceof IndexOutOfBoundsException);
        }

        // Object
        //xxx

        // conversions
        s = ja.getString(3);
        assertEquals("true", s);
        s = ja.getString(4);
        assertEquals("false", s);
        s = ja.getString(2);
        assertEquals("-39543", s);
        s = ja.getString(8);
        assertEquals("-23", s);
        s = ja.getString(6);
        assertEquals("3.9942", s);

        ja.put(11, "false");
        ja.put(12, "true");
        b = ja.getBoolean(11);
        assertEquals(false, b);
        b = ja.getBoolean(12);
        assertEquals(true, b);
        
        ja.put(13, "Sat, 21 Apr 2007 02:47:23.00 -0400");
        calendar.clear();
        calendar.setTimeZone(TimeZone.getTimeZone("GMT-0400"));
        calendar.set(2007, 3, 21, 2, 47, 23);
        Date d2 = calendar.getTime();
        date = ja.getDate(13);
        assertEquals(d2, date);

        ja.put(14, "97.22");
        d = ja.getDouble(14);
        assertEquals(97.22, d);

        i = ja.getInt(14);
        assertEquals(97, i);

        l = ja.getLong(14);
        assertEquals(97L, l);

        // failed conversions
        try {
            b = ja.getBoolean(9);
            fail("exception expected");
        }
        catch (Exception ex)
        {
            assertTrue(ex instanceof JSONException);
        }
        try {
            d = ja.getDouble(9);
            fail("exception expected");
        }
        catch (Exception ex)
        {
            assertTrue(ex instanceof JSONException);
        }
        try {
            i = ja.getInt(9);
            fail("exception expected");
        }
        catch (Exception ex)
        {
            assertTrue(ex instanceof JSONException);
        }
        try {
            l = ja.getLong(9);
            fail("exception expected");
        }
        catch (Exception ex)
        {
            assertTrue(ex instanceof JSONException);
        }

    }
    
    //xxx isnul, length, overwrite, add, addALl, remove
    @Test
    public void testJSONArrayMap()
    {
        
    }
    
}
