package org.json;

import static org.junit.Assert.*;

import java.util.Calendar;
import java.util.Date;
import java.util.NoSuchElementException;
import java.util.TimeZone;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

public class TestJSONObject
{

    @Before
    public void setUp() throws Exception
    {
    }

    @After
    public void tearDown() throws Exception
    {
    }

    @Test
    public void testNulls()
    {
        JSONObject jo = new JSONObject();
        jo.put("s1", (Object)null);
        jo.put("s2", "string");
        JSONArray a = new JSONArray();
        a.add("1");
        a.add(null);
        a.add("2");
        jo.put("a", a);
        //xxx System.out.println(t.toString());
        assertEquals("{\"s2\":\"string\",\"a\":[\"1\",null,\"2\"],\"s1\":null}", 
                jo.toString());

        assertNull(jo.get("s1"));
        assertTrue(jo.isNull("s1"));

        JSONArray at = jo.getJSONArray("a");
        assertNull(at.get(1));
        assertTrue(at.isNull(1));

        jo = new JSONObject(
            "{\"s2\":null,\"a\":[\"1\",null,\"2\",null,\"3\"],\"s3\":\"string\"}");

        try
        {
            jo.get("s1");
            fail("expecting exception");
        }
        catch (Exception ex)
        {
            // OK
        }
        assertNull(jo.opt("s1"));

        assertNull(jo.get("s2"));
        assertTrue(jo.isNull("s2"));

        at = jo.getJSONArray("a");
        assertNull(at.get(1));
        assertNull(at.get(3));
        assertTrue(at.isNull(1));
        assertTrue(at.isNull(3));
    }

    @Test
    public void testJSONObjectConstruct()
    {
        
    }

    @Test
    public void testJSONObjectGetSet()
    {
        JSONObject jo = new JSONObject();

        jo.put("n1", (Object)null);
        Object o = jo.get("n1");
        assertNull(o);
        o = jo.opt("n1");
        assertNull(o);

        // int
        JSONObject t = jo.put("int1", 0);
        assertEquals(t, jo);
        assertTrue(t == jo);
        jo.put("int2", -39543);
        int i = jo.getInt("int1");
        assertEquals(0, i);
        i = jo.getInt("int2");
        assertEquals(-39543, i);
        i = jo.optInt("int2");
        assertEquals(-39543, i);
        i = jo.optInt("int2", 93);
        assertEquals(-39543, i);
        i = jo.optInt("intMissing"); // default is 0
        assertEquals(0, i);
        i = jo.optInt("intMissing",7932);
        assertEquals(7932, i);
        try
        {
            i = jo.getInt("intMissing");
            fail("exception expected");
        }
        catch (Exception ex)
        {
            assertTrue(ex instanceof NoSuchElementException);
        }
        i = jo.optInt("n1");
        assertEquals(0, i);
        try
        {
            i = jo.getInt("n1");
            fail("exception expected");
        }
        catch (Exception ex)
        {
            assertTrue(ex instanceof JSONException);
        }

        // boolean
        jo.put("bool1", true);
        jo.put("bool2", false);
        boolean b = jo.getBoolean("bool1");
        assertEquals(true, b);
        b = jo.getBoolean("bool2");
        assertEquals(false, b);
        b = jo.optBoolean("bool2");
        assertEquals(false, b);
        b = jo.optBoolean("bool2", true);
        assertEquals(false, b);
        b = jo.optBoolean("boolMissing"); // default is false
        assertEquals(false, b);
        b = jo.optBoolean("boolMissing",true);
        assertEquals(true, b);
        try
        {
            b = jo.getBoolean("boolMissing");
            fail("exception expected");
        }
        catch (Exception ex)
        {
            assertTrue(ex instanceof NoSuchElementException);
        }
        b = jo.optBoolean("n1");
        assertEquals(false, b);
        try
        {
            b = jo.getBoolean("n1");
            fail("exception expected");
        }
        catch (Exception ex)
        {
            assertTrue(ex instanceof JSONException);
        }

        // double
        jo.put("d1", -2.87e15);
        jo.put("d2", 3.9942);
        double d = jo.getDouble("d1");
        assertEquals(-2.87e15, d);
        d = jo.getDouble("d2");
        assertEquals(3.9942, d);
        d = jo.optDouble("d2");
        assertEquals(3.9942, d);
        d = jo.optDouble("d2", 9.3);
        assertEquals(3.9942, d);
        d = jo.optDouble("dMissing"); // default is 0.0
        assertEquals(0.0, d);
        d = jo.optDouble("dMissing",-7.882);
        assertEquals(-7.882, d);
        try
        {
            d = jo.getDouble("dMissing");
            fail("exception expected");
        }
        catch (Exception ex)
        {
            assertTrue(ex instanceof NoSuchElementException);
        }
        d = jo.optDouble("n1");
        assertEquals(0.0, d);
        try
        {
            d = jo.getDouble("n1");
            fail("exception expected");
        }
        catch (Exception ex)
        {
            assertTrue(ex instanceof JSONException);
        }
        // bad numbers
        try {
            double bad = Double.NaN;
            jo.put("badd", bad);
            fail("exception expected");
        }
        catch (Exception ex)
        {
            assertTrue(ex instanceof JSONException);
        }
        try {
            Float bad = Float.NEGATIVE_INFINITY;
            jo.put("badf", bad);
            fail("exception expected");
        }
        catch (Exception ex)
        {
            assertTrue(ex instanceof JSONException);
        }

        // long
        jo.put("long1", 990001L);
        jo.put("long2", -23L);
        long l = jo.getLong("long1");
        assertEquals(990001L, l);
        l = jo.getLong("long2");
        assertEquals(-23L, l);
        l = jo.optLong("long2");
        assertEquals(-23L, l);
        l = jo.optLong("long2", 9L);
        assertEquals(-23L, l);
        l = jo.optLong("longMissing"); // default is 0
        assertEquals(0L, l);
        l = jo.optLong("longMissing", 7882L);
        assertEquals(7882L, l);
        try
        {
            l = jo.getLong("longMissing");
            fail("exception expected");
        }
        catch (Exception ex)
        {
            assertTrue(ex instanceof NoSuchElementException);
        }
        l = jo.optLong("n1");
        assertEquals(0L, l);
        try
        {
            l = jo.getLong("n1");
            fail("exception expected");
        }
        catch (Exception ex)
        {
            assertTrue(ex instanceof JSONException);
        }

        // String
        jo.put("str1", "abc").put("str2", "xyz");
        String s = jo.getString("str1");
        assertEquals("abc", s);
        s = jo.getString("str2");
        assertEquals("xyz", s);
        s = jo.optString("str2");
        assertEquals("xyz", s);
        s = jo.optString("str2", "junk");
        assertEquals("xyz", s);
        s = jo.optString("strMissing"); // default is ""
        assertEquals("", s);
        s = jo.optString("strMissing", "def");
        assertEquals("def", s);
        s = jo.getString("n1");
        assertEquals("null", s);
        s = jo.optString("n1");
        assertEquals("", s);

        try
        {
            l = jo.getLong("longMissing");
            fail("exception expected");
        }
        catch (Exception ex)
        {
            assertTrue(ex instanceof NoSuchElementException);
        }

        // Date
        Calendar calendar = Calendar.getInstance();
        calendar.clear();
        calendar.set(2006, 1, 2, 11, 32);
        Date d1 = calendar.getTime();
        Date now = new Date();

        jo.put("date1", d1);
        Date date = jo.getDate("date1");
        assertEquals(d1, date);
        date = jo.optDate("date1");
        assertEquals(d1, date);
        date = jo.optDate("date1", now);
        assertEquals(d1, date);
        date = jo.optDate("dateMissing"); // default is current date so can't assert much
        date = jo.optDate("dateMissing", now);
        assertEquals(now, date);
        try
        {
            date = jo.getDate("dateMissing");
            fail("exception expected");
        }
        catch (Exception ex)
        {
            assertTrue(ex instanceof NoSuchElementException);
        }

        // Object
        //xxx

        // conversions
        s = jo.getString("bool1");
        assertEquals("true", s);
        s = jo.getString("bool2");
        assertEquals("false", s);
        s = jo.getString("int2");
        assertEquals("-39543", s);
        s = jo.getString("long2");
        assertEquals("-23", s);
        s = jo.getString("d2");
        assertEquals("3.9942", s);

        jo.put("bool3", "false");
        jo.put("bool4", "true");
        b = jo.getBoolean("bool3");
        assertEquals(false, b);
        b = jo.getBoolean("bool4");
        assertEquals(true, b);
        
        jo.put("date2", "Sat, 21 Apr 2007 02:47:23.00 -0400");
        calendar.clear();
        calendar.setTimeZone(TimeZone.getTimeZone("GMT-0400"));
        calendar.set(2007, 3, 21, 2, 47, 23);
        Date d2 = calendar.getTime();
        date = jo.getDate("date2");
        assertEquals(d2, date);

        jo.put("d3", "97.22");
        d = jo.getDouble("d3");
        assertEquals(97.22, d);

        i = jo.getInt("d3");
        assertEquals(97, i);

        l = jo.getLong("d3");
        assertEquals(97L, l);

        // failed conversions
        try {
            b = jo.getBoolean("str1");
            fail("exception expected");
        }
        catch (Exception ex)
        {
            assertTrue(ex instanceof JSONException);
        }
        try {
            d = jo.getDouble("str1");
            fail("exception expected");
        }
        catch (Exception ex)
        {
            assertTrue(ex instanceof JSONException);
        }
        try {
            i = jo.getInt("str1");
            fail("exception expected");
        }
        catch (Exception ex)
        {
            assertTrue(ex instanceof JSONException);
        }
        try {
            l = jo.getLong("str1");
            fail("exception expected");
        }
        catch (Exception ex)
        {
            assertTrue(ex instanceof JSONException);
        }
        //xxx

        // null keys
        try
        {
            jo.put(null, 89);
            fail("exception expected");
        }
        catch (Exception ex)
        {
            assertTrue(ex instanceof NullPointerException);
        }
        try
        {
            o = jo.get(null);
            fail("exception expected");
        }
        catch (Exception ex)
        {
            assertTrue(ex instanceof NoSuchElementException);
        }
        o = jo.opt(null);
        assertNull(o);
    }
    
    //xxx has, isnul, keys, length remove, containsKey, containsValue, putAll, put
    @Test
    public void testJSONObjectMap()
    {
        
    }
    
}
